/**
 * @license
 * Copyright 2021 Google LLC
 * SPDX-License-Identifier: Apache-2.0
 */
import { LitElement } from 'lit';
import { ARIARole } from '../../internal/aria/aria.js';
import { ListItem } from './listitem/list-item.js';
/**
 * A record that describes a list item in a list with metadata such a reference
 * to the item and its index in the list.
 */
export interface ItemRecord {
    item: ListItem;
    index: number;
}
export declare class List extends LitElement {
    /** @nocollapse */
    static shadowRootOptions: ShadowRootInit;
    type: ARIARole | '';
    /**
     * The tabindex of the underlying list.
     */
    listTabIndex: number;
    private listRoot;
    /**
     * An array of activatable and disableable list items. Queries every assigned
     * element that has the `md-list-item` attribute.
     *
     * _NOTE:_ This is a shallow, flattened query via
     * `HTMLSlotElement.queryAssignedElements` and thus will _only_ include direct
     * children / directly slotted elements.
     */
    items: ListItem[];
    protected render(): import("lit-html").TemplateResult<1>;
    /**
     * Renders the main list element.
     */
    private renderList;
    /**
     * The content to be slotted into the list.
     */
    private renderContent;
    /**
     * Handles keyboard navigation in the list.
     *
     * @param event {KeyboardEvent} The keyboard event that triggers this handler.
     */
    private handleKeydown;
    private activateNextItemInternal;
    private activatePreviousItemInternal;
    /**
     * Activates the next item in the list. If at the end of the list, the first
     * item will be activated.
     *
     * @return The activated list item or `null` if there are no items.
     */
    activateNextItem(): ListItem | null;
    /**
     * Activates the previous item in the list. If at the start of the list, the
     * last item will be activated.
     *
     * @return The activated list item or `null` if there are no items.
     */
    activatePreviousItem(): ListItem | null;
    /**
     * Activates the first non-disabled item of a given array of items.
     *
     * @param items {Array<ListItem>} The items from which to activate the
     * first item.
     * @nocollapse
     */
    static activateFirstItem<T extends ListItem>(items: T[]): T;
    /**
     * Activates the last non-disabled item of a given array of items.
     *
     * @param items {Array<ListItem>} The items from which to activate the
     * last item.
     * @nocollapse
     */
    static activateLastItem<T extends ListItem>(items: T[]): T;
    /**
     * Deactivates the currently active item of a given array of items.
     *
     * @param items {Array<ListItem>} The items from which to deactivate the
     * active item.
     * @return A record of the deleselcted activated item including the item and
     * the index of the item or `null` if none are deactivated.
     * @nocollapse
     */
    static deactivateActiveItem<T extends ListItem>(items: T[]): ItemRecord;
    focus(): void;
    /**
     * Retrieves the first activated item of a given array of items.
     *
     * @param items {Array<ListItem>} The items to search.
     * @return A record of the first activated item including the item and the
     * index of the item or `null` if none are activated.
     * @nocollapse
     */
    static getActiveItem<T extends ListItem>(items: T[]): ItemRecord;
    /**
     * Retrieves the first non-disabled item of a given array of items. This
     * the first item that is not disabled.
     *
     * @param items {Array<ListItem>} The items to search.
     * @return The first activatable item or `null` if none are activatable.
     * @nocollapse
     */
    static getFirstActivatableItem<T extends ListItem>(items: T[]): T;
    /**
     * Retrieves the last non-disabled item of a given array of items.
     *
     * @param items {Array<ListItem>} The items to search.
     * @return The last activatable item or `null` if none are activatable.
     * @nocollapse
     */
    static getLastActivatableItem<T extends ListItem>(items: T[]): T;
    /**
     * Retrieves the next non-disabled item of a given array of items.
     *
     * @param items {Array<ListItem>} The items to search.
     * @param index {{index: number}} The index to search from.
     * @return The next activatable item or `null` if none are activatable.
     */
    private static getNextItem;
    /**
     * Retrieves the previous non-disabled item of a given array of items.
     *
     * @param items {Array<ListItem>} The items to search.
     * @param index {{index: number}} The index to search from.
     * @return The previous activatable item or `null` if none are activatable.
     */
    private static getPrevItem;
}
